@@ -1,6 +1,6 @@
 ;;; rec-mode.el --- Major mode for viewing/editing rec files
 
-;; Copyright (C) 2009 Jose E. Marchesi
+;; Copyright (C) 2009, 2010, 2011 Jose E. Marchesi
 
 ;; Maintainer: Jose E. Marchesi
 
@@ -101,7 +101,7 @@ Valid values are `edit' and `navigation'.  The default is `navigation'"
   "Syntax table used in rec-mode")
 
 (defvar rec-font-lock-keywords
-  `(("^%\\(rec\\|key\\|unique\\|type\\|prohibit\\|mandatory\\|type\\|doc\\|fsort|auto\\):" . font-lock-keyword-face)
+  `(("^%\\(rec\\|key\\|unique\\|type\\|typedef\\|prohibit\\|mandatory\\|type\\|doc\\|fsort\\|auto\\|confidential\\|sort\\):" . font-lock-keyword-face)
     (,rec-field-name-re . font-lock-variable-name-face)
     ("^\\+" . font-lock-constant-face))
   "Font lock keywords used in rec-mode")
