@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2009, 2010 Jose E. Marchesi */
+/* Copyright (C) 2009, 2010, 2011, 2012 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include <gettext.h>
 #define _(str) dgettext (PACKAGE, str)
 
+#include <rec-utils.h>
 #include <rec.h>
 
 /* Field name Data Structure
@@ -50,6 +51,34 @@ struct rec_field_name_s
   int size;
 };
 
+/* The following global variable contains field names which are either
+   standard, such as the special field names, or often used in the
+   library.
+   
+   Note that the array is indexed using the rec_std_field_e enumerated
+   values defined in rec.h, so the order is important.  */
+
+static struct rec_field_name_s fnames[] =
+  {
+    {{"%auto", NULL, NULL}, 1},
+    {{"%confidential", NULL, NULL}, 1},
+    {{"%key", NULL, NULL}, 1},
+    {{"%mandatory", NULL, NULL}, 1},
+    {{"%prohibit", NULL, NULL}, 1},
+    {{"%rec", NULL, NULL}, 1},
+    {{"%size", NULL, NULL}, 1},
+    {{"%sort", NULL, NULL}, 1},
+    {{"%type", NULL, NULL}, 1},
+    {{"%typedef", NULL, NULL}, 1},
+    {{"%unique", NULL, NULL}, 1}
+  };
+
+rec_field_name_t
+rec_std_field_name (enum rec_std_field_e std_field)
+{
+  return (fnames + std_field);
+}
+
 rec_field_name_t
 rec_field_name_new ()
 {
@@ -236,43 +265,13 @@ rec_field_name_ref_p (rec_field_name_t fname1,
 bool
 rec_field_name_str_p (const char *str)
 {
-  int ret;
-  regex_t regexp;
-
-  /* Compile the regexp.  */
-  if ((ret = regcomp (&regexp, "^" REC_FNAME_RE "$", REG_EXTENDED)) != 0)
-    {
-      fprintf (stderr, _("internal error: rec_field_name_str_p: error compiling regexp.\n"));
-      return false;
-    }
-
-  /* Check.  */
-  ret = regexec (&regexp, str, 0, NULL, 0);
-
-  regfree (&regexp);
-  return (ret == 0);
+  return rec_match (str, "^" REC_FNAME_RE "$");
 }
 
 bool
 rec_field_name_part_str_p (const char *str)
 {
-  int ret;
-  regex_t regexp;
-
-  /* Compile the regexp.  */
-  if ((ret = regcomp (&regexp,
-                      "^" REC_FNAME_PART_RE "$",
-                      REG_EXTENDED)) != 0)
-    {
-      fprintf (stderr, _("internal error: rec_field_name_part_str_p: error compiling regexp.\n"));
-      return false;
-    }
-
-  /* Check.  */
-  ret = regexec (&regexp, str, 0, NULL, 0);
-
-  regfree (&regexp);
-  return (ret == 0);
+  return rec_match (str, "^" REC_FNAME_PART_RE "$");
 }
 
 char *
