@@ -72,14 +72,16 @@ static bool rec_int_rec_type_p (char *str);
 
 #define FNAME(id) rec_std_field_name ((id))
 
-#define ADD_ERROR(buf,str,...)                  \
-  do                                            \
-    {                                           \
-       char *tmp = NULL;                        \
-       asprintf (&tmp, (str), __VA_ARGS__);     \
-       rec_buf_puts (tmp, (buf));               \
-       free (tmp);                              \
-    }                                           \
+#define ADD_ERROR(buf,str,...)                          \
+  do                                                    \
+    {                                                   \
+      char *tmp = NULL;                                 \
+      if (asprintf (&tmp, (str), __VA_ARGS__) != -1)    \
+        {                                               \
+          rec_buf_puts (tmp, (buf));                    \
+          free (tmp);                                   \
+        }                                               \
+    }                                                   \
   while (0)
 
 /*
@@ -1023,6 +1025,7 @@ rec_int_merge_remote (rec_rset_t rset,
             {
 #if defined REMOTE_DESCRIPTORS
               CURL *curl;
+              int tmpfile_des;
 
               /* Fetch the remote descriptor.  */
               curl = curl_easy_init ();
