@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2010 Jose E. Marchesi */
+/* Copyright (C) 2010, 2011 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -198,7 +198,7 @@ rec_sex_eval (rec_sex_t sex,
   rec_field_t field;
   rec_field_t wfield;
   rec_record_t wrec;
-  rec_record_elem_t elem_field;
+  rec_mset_iterator_t iter;
   int j, nf;
   struct rec_sex_val_s val;
   
@@ -212,11 +212,9 @@ rec_sex_eval (rec_sex_t sex,
       goto exit;
     }
 
-  elem_field = rec_record_null_elem ();
-  while (rec_record_elem_p (elem_field = rec_record_next_field (record, elem_field)))
+  iter = rec_mset_iterator (rec_record_mset (record));
+  while (rec_mset_iterator_next (&iter, MSET_FIELD, (const void**) &field, NULL))
     {
-      field = rec_record_elem_field (elem_field);
-
       nf = rec_record_get_num_fields_by_name (record, rec_field_name (field));
       if (nf > 1)
         /* XXX.  Optimization: && (rec_sex_ast_name_p (sex->ast, field_name_str))) */
@@ -235,7 +233,7 @@ rec_sex_eval (rec_sex_t sex,
               rec_record_remove_field_by_name (wrec,
                                                rec_field_name (field),
                                                -1); /* Delete all.  */
-              rec_record_append_field (wrec, rec_field_dup (wfield));
+              rec_mset_append (rec_record_mset (wrec), MSET_FIELD, (void *) rec_field_dup (wfield), MSET_ANY);
 
               EXEC_AST(wrec);
 
@@ -248,6 +246,8 @@ rec_sex_eval (rec_sex_t sex,
         }
     }
 
+  rec_mset_iterator_free (&iter);
+
  exit:          
 
   if (!*status)
@@ -657,31 +657,18 @@ rec_sex_eval_node (rec_sex_t sex,
             && (child_val2.type == REC_SEX_VAL_STR))
           {
             /* String match.  */
-            regex_t regexp;
-            int flags;
+            res.type = REC_SEX_VAL_INT;
 
-            flags = REG_EXTENDED;
             if (rec_sex_parser_case_insensitive (sex->parser))
               {
-                flags |= REG_ICASE;
+                res.int_val =
+                  rec_match_insensitive (child_val1.str_val, child_val2.str_val);
               }
-
-            if (regcomp (&regexp,
-                         child_val2.str_val,
-                         flags) != 0)
+            else
               {
-                *status = false;
-                return res;
+                res.int_val =
+                  rec_match (child_val1.str_val, child_val2.str_val);
               }
-
-            res.type = REC_SEX_VAL_INT;
-            res.int_val = (regexec (&regexp,
-                                    child_val1.str_val,
-                                    0,
-                                    NULL,
-                                    0) == 0);
-
-            regfree (&regexp);
           }
         else
           {
