@@ -41,9 +41,13 @@ enum rec_size_condition_e
     SIZE_COND_GE
   };
 
-/* Non public constants used by several modules.  */
+/* Non public constants used by several modules. */
 
-#define REC_INT_SIZE_RE "^[ \t\n]*(>|<|>=|<=)?[ \t\n]*([0-9]+)[ \t\n]*$"
+#define REC_INT_RE "-?((0x[0-9a-fA-F]+)|[0-9]+)"
+#define REC_INT_SIZE_RE                         \
+  "^[ \t\n]*(>|<|>=|<=)?[ \t\n]*"               \
+  REC_INT_RE                                    \
+  "[ \t\n]*$"
 
 /* Parse an integer/real in the NULL-terminated string STR and store
    it at NUMBER.  Return true if the conversion was successful.  false
@@ -62,6 +66,7 @@ enum rec_size_condition_e rec_extract_size_condition (const char *str);
 
 /* Matching a string against a regexp.  */
 bool rec_match (const char *str, const char *regexp);
+bool rec_match_insensitive (const char *str, const char *regexp);
 
 /* Generic parsing routines.  */
 bool rec_blank_p (char c);
