@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2009, 2010, 2011 Jose E. Marchesi */
+/* Copyright (C) 2009, 2010, 2011, 2012 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,6 +54,16 @@ struct rec_writer_s
                         fields.  */
 };
 
+static void
+rec_writer_new_common (rec_writer_t writer)
+{
+  writer->file_out = NULL;
+  writer->buf_out = NULL;
+  writer->line = 1;
+  writer->eof = false;
+  writer->password = NULL;
+}
+
 rec_writer_t
 rec_writer_new (FILE *file_out)
 {
@@ -62,11 +72,8 @@ rec_writer_new (FILE *file_out)
   new = malloc (sizeof(struct rec_writer_s));
   if (new)
     {
+      rec_writer_new_common (new);
       new->file_out = file_out;
-      new->buf_out = NULL;
-      new->line = 1;
-      new->eof = false;
-      new->password = NULL;
     }
 
   return new;
@@ -80,10 +87,8 @@ rec_writer_new_str (char **str, size_t *str_size)
   new = malloc (sizeof(struct rec_writer_s));
   if (new)
     {
-      new->file_out = NULL;
+      rec_writer_new_common (new);
       new->buf_out = rec_buf_new (str, str_size);
-      new->line = 1;
-      new->eof = false;
     }
 
   return new;
@@ -222,14 +227,14 @@ rec_write_field_with_rset (rec_writer_t writer,
       return false;
     }
   
-  if (!rec_writer_putc (writer, ' '))
-    {
-      return false;
-    }
-  
   /* Write the field value */
   if (mode == REC_WRITER_SEXP)
     {
+      if (!rec_writer_putc (writer, ' '))
+        {
+          return false;
+        }
+  
       if (!rec_writer_putc (writer, '"'))
         {
           return false;
@@ -244,6 +249,15 @@ rec_write_field_with_rset (rec_writer_t writer,
 #endif /* REC_CRYPT_SUPPORT */
 
   fvalue = rec_field_value (field);
+
+  if ((strlen (fvalue) > 0) && (mode != REC_WRITER_SEXP))
+    {
+      if (!rec_writer_putc (writer, ' '))
+        {
+          return false;
+        }
+    }
+
   for (pos = 0; pos < strlen (fvalue); pos++)
     {
       if (fvalue[pos] == '\n')
@@ -392,7 +406,9 @@ rec_write_record_with_rset (rec_writer_t writer,
   bool ret;
   rec_field_t field;
   rec_comment_t comment;
-  rec_record_elem_t elem;
+  rec_mset_iterator_t iter;
+  rec_mset_elem_t elem;
+  char *data;
 
   ret = true;
 
@@ -412,14 +428,13 @@ rec_write_record_with_rset (rec_writer_t writer,
         }
     }
 
-
-  elem = rec_record_null_elem ();
-  while (rec_record_elem_p (elem = rec_record_next (record, elem)))
+  iter = rec_mset_iterator (rec_record_mset (record));
+  while (rec_mset_iterator_next (&iter, MSET_ANY, (const void **) &data, &elem))
     {
-      if (rec_record_elem_field_p (record, elem))
+      if (rec_mset_elem_type (elem) == MSET_FIELD)
         {
           /* Write a field.  */
-          field = rec_record_elem_field (elem);
+          field = (rec_field_t) data;
           if (!rec_write_field_with_rset (writer,
                                           rset,
                                           field,
@@ -432,7 +447,7 @@ rec_write_record_with_rset (rec_writer_t writer,
       else
         {
           /* Write a comment.  */
-          comment = rec_record_elem_comment (elem);
+          comment = (rec_comment_t) data;
           if (!rec_write_comment (writer, comment, mode))
             {
               ret = false;
@@ -441,6 +456,8 @@ rec_write_record_with_rset (rec_writer_t writer,
         }
     }
 
+  rec_mset_iterator_free (&iter);
+
   if (mode == REC_WRITER_SEXP)
     {
       if (!rec_writer_puts (writer, "))\n"))
@@ -502,6 +519,13 @@ rec_write_record_with_fex (rec_writer_t writer,
 
           written_fields++;
 
+#if defined REC_CRYPT_SUPPORT
+          if (writer->password)
+            {
+              rec_decrypt_field (field, writer->password);
+            }
+#endif /* REC_CRYPT_SUPPORT */
+
           if (print_values_p)
             {
               /* Write just the value of the field.  */
@@ -544,10 +568,12 @@ rec_write_rset (rec_writer_t writer,
   bool ret;
   rec_record_t descriptor;
   bool wrote_descriptor;
-  rec_rset_elem_t elem;
   size_t position;
   size_t descriptor_pos;
-
+  rec_mset_iterator_t iter;
+  rec_mset_elem_t elem;
+  void *data;
+  
   ret = true;
   wrote_descriptor = false;
   position = 0;
@@ -564,8 +590,8 @@ rec_write_rset (rec_writer_t writer,
       return true;
     }
 
-  elem = rec_rset_null_elem ();
-  while (rec_rset_elem_p (elem = rec_rset_next (rset, elem)))
+  iter = rec_mset_iterator (rec_rset_mset (rset));
+  while (rec_mset_iterator_next (&iter, MSET_ANY, (const void **) &data, &elem))
     {
       if (position != 0)
         {
@@ -596,16 +622,16 @@ rec_write_rset (rec_writer_t writer,
             }
         }
       
-      if (rec_rset_elem_record_p (rset, elem))
+      if (rec_mset_elem_type (elem) == MSET_RECORD)
         {
           ret = rec_write_record (writer,
-                                  rec_rset_elem_record (elem),
+                                  (rec_record_t) data,
                                   REC_WRITER_NORMAL);
         }
       else
         {
           ret = rec_write_comment (writer,
-                                   rec_rset_elem_comment (elem),
+                                   (rec_comment_t) data,
                                    REC_WRITER_NORMAL);
         }
       
@@ -617,6 +643,8 @@ rec_write_rset (rec_writer_t writer,
       position++;
     }
 
+  rec_mset_iterator_free (&iter);
+
   /* Special case:
    *
    * # comment 1
