@@ -2,7 +2,7 @@
 #
 # recdel.sh - System tests for recdel.
 #
-# Copyright (C) 2010 Jose E. Marchesi.
+# Copyright (C) 2010, 2011, 2012 Jose E. Marchesi.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -222,6 +222,30 @@ test_tool recdel-no-external ok \
 Id: bar
 '
 
+test_tool recdel-quick-simple ok \
+          recdel \
+          '-q value22' \
+          multiple-records \
+'field1: value11
+field2: value12
+field3: value13
+
+field1: value31
+field2: value32
+field3: value33
+'
+
+test_tool recdel-random-all xfail \
+          recdel \
+          '-m 0' \
+          multiple-records
+
+test_tool recdel-random-all-force ok \
+          recdel \
+          '-m 0 --force' \
+          multiple-records \
+''
+
 #
 # Cleanup
 #
