@@ -2,7 +2,7 @@
 #
 # recins.sh - System tests for recins.
 #
-# Copyright (C) 2010 Jose E. Marchesi.
+# Copyright (C) 2010, 2011, 2012 Jose E. Marchesi.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,6 +32,12 @@ test_init "recins"
 test_declare_input_file empty-file \
 ''
 
+test_declare_input_file one-record \
+'field1: value11
+field2: value12
+field3: value13
+'
+
 test_declare_input_file multiple-records \
 'field1: value11
 field2: value12
@@ -106,6 +112,13 @@ myint: 10
 myrange: 0
 '
 
+test_declare_input_file auto-notype \
+'%rec: foo
+%auto: key
+
+key: 12
+'
+
 test_declare_input_file auto-fields-several \
 '%rec: foo
 %type: myint int
@@ -233,6 +246,32 @@ field2: value32
 field3: value33
 '
 
+test_tool recins-replace-range ok \
+          recins \
+          '-n 1-2 -f afield1 -v "replaced"' \
+          multiple-records \
+'field1: value11
+field2: value12
+field3: value13
+
+afield1: replaced
+
+afield1: replaced
+'
+
+test_tool recins-replace-several-indexes ok \
+          recins \
+          '-n 0,2 -f afield1 -v "replaced"' \
+          multiple-records \
+'afield1: replaced
+
+field1: value21
+field2: value22
+field3: value23
+
+afield1: replaced
+'
+
 test_tool recins-replace-sex ok \
           recins \
           '-e "field2 = '\''value22'\''" -f afield1 -v "replaced"' \
@@ -311,6 +350,19 @@ myrange: 1
 foo: bar
 '
 
+test_tool recins-auto-notype ok \
+          recins \
+          '-t foo -f foo -v bar' \
+          auto-notype \
+'%rec: foo
+%auto: key
+
+key: 12
+
+key: 13
+foo: bar
+'
+
 test_tool recins-auto-fields-several ok \
           recins \
           '-t foo -f foo -v bar' \
@@ -362,6 +414,47 @@ myrange: 10
 myrange: 11
 foo: bar
 '
+
+test_tool recins-quick-simple ok \
+          recins \
+          '-q value22 -f foo -v bar' \
+          multiple-records \
+'field1: value11
+field2: value12
+field3: value13
+
+foo: bar
+
+field1: value31
+field2: value32
+field3: value33
+'
+
+test_tool recins-random-all ok \
+          recins \
+          '-m 0 -f foo -v bar' \
+          multiple-records \
+'field1: value11
+field2: value12
+field3: value13
+
+field1: value21
+field2: value22
+field3: value23
+
+field1: value31
+field2: value32
+field3: value33
+
+foo: bar
+'
+
+test_tool recins-random-one ok \
+          recins \
+          '-m 1 -f foo -v bar' \
+          one-record \
+'foo: bar
+'
  
 #
 # Cleanup.
