@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2009, 2010, 2011 Jose E. Marchesi */
+/* Copyright (C) 2009, 2010, 2011, 2012 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,12 +46,13 @@ void recdel_parse_args (int argc, char **argv);
 char *recutl_type = NULL;
 bool recdel_comment = false;
 rec_sex_t recutl_sex = NULL;
+char *recutl_quick_str = NULL;
 char *recutl_sex_str = NULL;
-int recutl_num = -1;
 bool recutl_insensitive = false;
 bool recdel_force = false;
 bool recdel_verbose = false;
 bool recdel_external = true;
+size_t recutl_random = 0;
 char *recdel_file = NULL;  /* File from where to delete the
                               records.  */
 
@@ -86,7 +87,7 @@ recutl_print_help (void)
   /* TRANSLATORS: --help output, recdel synopsis.
      no-wrap */
   printf (_("\
-Usage: recdel [OPTIONS]... [-t TYPE] [-n NUM | -e EXPR] [FILE]\n"));
+Usage: recdel [OPTIONS]... [-t TYPE] [-n NUM | -e EXPR | -q STR | -m NUM] [FILE]\n"));
 
   /* TRANSLATORS: --help output, recdel short description.
      no-wrap */
@@ -134,8 +135,8 @@ recdel_delete_records (rec_db_t db)
   rec_record_t record;
   rec_comment_t comment;
   bool parse_status = true;
-  rec_rset_elem_t rec_elem;
-  rec_rset_elem_t new_elem;
+  rec_mset_iterator_t iter;
+  rec_mset_elem_t elem;
 
   if (!rec_db_type_p (db, recutl_type))
     {
@@ -177,31 +178,45 @@ recdel_delete_records (rec_db_t db)
 
       /* Process this record set.  */
       numrec = 0;
-      rec_elem = rec_rset_first_record (rset);
-      while (rec_rset_elem_p (rec_elem))
+
+      /* If the user requested to delete random records, calculate
+         them now for this record set.  */
+
+      if (recutl_random > 0)
         {
-          record = rec_rset_elem_record (rec_elem);
-          
-          if (((recutl_num == -1) && !recutl_sex)
-              || (((recutl_num == -1) &&
-                   (recutl_sex &&
-                    (rec_sex_eval (recutl_sex, record, &parse_status))))
-                  || (recutl_num == numrec)))
+          recutl_reset_indexes ();
+          recutl_index_add_random (recutl_random, rec_rset_num_records (rset));
+        }
+
+      iter = rec_mset_iterator (rec_rset_mset (rset));
+      while (rec_mset_iterator_next (&iter, MSET_RECORD, (const void **) &record, &elem))
+        {
+          if ((recutl_quick_str && rec_record_contains_value (record,
+                                                              recutl_quick_str,
+                                                              recutl_insensitive))
+              || (!recutl_quick_str && (((recutl_num_indexes() == 0) && !recutl_sex)
+                                        || (((recutl_num_indexes() == 0) &&
+                                             (recutl_sex &&
+                                              (rec_sex_eval (recutl_sex, record, &parse_status))))
+                                            || (recutl_index_p (numrec))))))
             {
-              /* Delete this record.  */
               if (recdel_comment)
                 {
-                  comment = rec_record_to_comment (record);
-                  new_elem = rec_rset_elem_comment_new (rset, comment);
-                  rec_rset_insert_after (rset, rec_elem, new_elem);
-                }
+                  /* Replace the records with a comment in the current
+                     element.  */
 
-              rec_elem = rec_rset_remove_record (rset, rec_elem);
-            }
-          else
-            {
-              /* Process the next record. */
-              rec_elem = rec_rset_next_record (rset, rec_elem);
+                  comment = rec_record_to_comment (record);
+                  rec_record_destroy (record);
+                  rec_mset_elem_set_data (elem, (void *) comment);
+                  rec_mset_elem_set_type (elem, MSET_COMMENT);
+                }
+              else
+                {
+                  /* Remove the record from the list and dispose
+                     it.  */
+
+                  rec_mset_remove_elem (rec_rset_mset (rset), elem);
+                }
             }
           
           if (!parse_status)
@@ -211,6 +226,8 @@ recdel_delete_records (rec_db_t db)
           
           numrec++;
         }
+
+      rec_mset_iterator_free (&iter);
     }
 
   /* Integrity check.  */
@@ -267,7 +284,7 @@ recdel_parse_args (int argc,
         }
     }
   
-  if ((recutl_num == -1) && !recutl_sex_str && !recdel_force)
+  if ((recutl_num_indexes() == 0) && !recutl_sex_str && !recutl_quick_str && !recdel_force && (recutl_random == 0))
     {
       recutl_error (_("ignoring a request to delete all records of type %s.\n"),
                     recutl_type ? recutl_type : "unknown");
@@ -311,7 +328,7 @@ main (int argc, char *argv[])
       recutl_fatal (_("cannot read file %s\n"), recdel_file);
     }
 
-  if (((recutl_num != -1) || recutl_sex) || recdel_force)
+  if (((recutl_num_indexes() != 0) || recutl_sex || recutl_quick_str) || recdel_force || (recutl_random > 0))
     {
       recdel_delete_records (db);
     }
