@@ -253,31 +253,32 @@ recfmt_apply_template (rec_record_t record,
 void
 recfmt_process_db (rec_db_t db, char *template)
 {
-  int rset_size;
-  size_t n_rset, n_rec;
+  size_t n_rset;
   rec_rset_t rset;
-  rec_record_t rec;
+  rec_record_t record;
   char *result;
+  rec_mset_iterator_t iter;
 
   /* Iterate on all the record sets.  */
   for (n_rset = 0; n_rset < rec_db_size (db); n_rset++)
     {
       rset = rec_db_get_rset (db, n_rset);
-      rset_size = rec_rset_num_records (rset);
 
       /* Iterate on all the records.  */
-      for (n_rec = 0; n_rec < rset_size; n_rec++)
-        {
-          rec = rec_rset_elem_record (rec_rset_get_record (rset, n_rec));
 
+      iter = rec_mset_iterator (rec_rset_mset (rset));
+      while (rec_mset_iterator_next (&iter, MSET_RECORD, (const void**) &record, NULL))
+        {
           /* Apply the template to this record.  */
-          result = recfmt_apply_template (rec, template);
+          result = recfmt_apply_template (record, template);
           if (result && (*result != '\0'))
             {
               puts (result);
               free (result);
             }
         }
+
+      rec_mset_iterator_free (&iter);
     }
 }
 
@@ -287,7 +288,6 @@ main (int argc, char *argv[])
   rec_db_t db;
 
   recutl_init ("recfmt");
-  recutl_sorting_parser (true, NULL, NULL);
 
   recfmt_parse_args (argc, argv);
 
