@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2010, 2011 Jose E. Marchesi */
+/* Copyright (C) 2010, 2011, 2012 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,6 +60,7 @@ static void recset_process_del (rec_rset_t rset, rec_record_t record, rec_fex_t
 
 char      *recutl_sex_str     = NULL;
 rec_sex_t  recutl_sex         = NULL;
+char      *recutl_quick_str   = NULL;
 char      *recutl_fex_str     = NULL;
 rec_fex_t  recutl_fex         = NULL;
 char      *recutl_type        = NULL;
@@ -67,12 +68,12 @@ int        recset_action      = RECSET_ACT_NONE;
 char      *recset_value       = NULL;
 rec_field_name_t recset_new_field_name = NULL;
 bool       recutl_insensitive = false;
-long       recutl_num         = -1;
 char      *recset_file        = NULL;
 bool       recset_force       = false;
 bool       recset_verbose     = false;
-bool       recset_external      = true;
+bool       recset_external    = true;
 bool       recset_descriptor_renamed = false;
+size_t     recutl_random      = 0;
 
 /*
  * Command line options management
@@ -331,7 +332,7 @@ recset_process_actions (rec_db_t db)
   rec_rset_t rset;
   rec_record_t record;
   bool parse_status = true;
-  rec_rset_elem_t rec_elem;
+  rec_mset_iterator_t iter;
   rec_buf_t errors_buf;
   char *errors_str;
   size_t errors_str_size;
@@ -373,16 +374,26 @@ recset_process_actions (rec_db_t db)
       /* Process this record set.  */
       numrec = 0;
 
-      rec_elem = rec_rset_first_record (rset);
-      while (rec_rset_elem_p (rec_elem))
+      /* If the user requested to process random records, calculate
+         them now for this record set.  */
+
+      if (recutl_random > 0)
         {
-          record = rec_rset_elem_record (rec_elem);
-          
-          if (((recutl_num == -1) && !recutl_sex)
-              || (((recutl_num == -1) &&
-                  ((recutl_sex &&
-                    (rec_sex_eval (recutl_sex, record, &parse_status)))))
-                  || (recutl_num == numrec)))
+          recutl_reset_indexes ();
+          recutl_index_add_random (recutl_random, rec_rset_num_records (rset));
+        }
+
+      iter = rec_mset_iterator (rec_rset_mset (rset));
+      while (rec_mset_iterator_next (&iter, MSET_RECORD, (const void**) &record, NULL))
+        {
+          if ((recutl_quick_str && rec_record_contains_value (record,
+                                                              recutl_quick_str,
+                                                              recutl_insensitive))
+              || (!recutl_quick_str && (((recutl_num_indexes() == 0) && !recutl_sex)
+                                        || (((recutl_num_indexes() == 0) &&
+                                             ((recutl_sex &&
+                                               (rec_sex_eval (recutl_sex, record, &parse_status)))))
+                                            || (recutl_index_p (numrec))))))
             {
               /* Process a copy of this record.  */
               switch (recset_action)
@@ -416,9 +427,6 @@ recset_process_actions (rec_db_t db)
                 }
             }
           
-          /* Process the next record.  */
-          rec_elem = rec_rset_next_record (rset, rec_elem);
-
           if (!parse_status)
             {
               recutl_fatal (_("invalid selection expression.\n"));
@@ -427,6 +435,8 @@ recset_process_actions (rec_db_t db)
           numrec++;
         }
 
+      rec_mset_iterator_free (&iter);
+
       /* Check for integrity in the modified rset.  */
       if (!recset_force)
         {
@@ -467,8 +477,7 @@ recset_process_add (rec_rset_t rset,
       field_name = rec_fex_elem_field_name (fex_elem);
       field = rec_field_new (rec_field_name_dup (field_name), recset_value);
 
-      /* XXX: sort the record afterwards.  */
-      rec_record_append_field (record, field);
+      rec_mset_append (rec_record_mset (record), MSET_FIELD, (void *) field, MSET_ANY);
     }
 }
 
@@ -524,7 +533,7 @@ recset_process_set (rec_rset_t rset,
         {
           /* Add a field with this name and value.  */
           field = rec_field_new (field_name, recset_value);
-          rec_record_append_field (record, field);
+          rec_mset_append (rec_record_mset (record), MSET_FIELD, (void *) field, MSET_ANY);
         }
     }
 }
@@ -584,7 +593,7 @@ recset_process_ren (rec_rset_t rset,
      But make sure to do it just once.
   */
   if ((!recset_descriptor_renamed)
-      && (recutl_sex == NULL) && (recutl_num == -1))
+      && (recutl_sex == NULL) && (recutl_num_indexes() == 0) && (recutl_random == 0) && (recutl_quick_str == NULL))
     {
       rec_rset_rename_field (rset,
                              field_name,
@@ -606,8 +615,8 @@ recset_process_del (rec_rset_t rset,
   rec_field_t field;
   rec_field_name_t field_name;
   rec_comment_t comment;
-  rec_record_elem_t comment_elem;
-  rec_record_elem_t field_elem;
+  rec_mset_iterator_t iter;
+  rec_mset_elem_t elem;
 
   /* Initialize the deletion mask.  */
   deletion_mask = xmalloc (sizeof (bool) * rec_record_num_fields (record));
@@ -652,27 +661,33 @@ recset_process_del (rec_rset_t rset,
                     
   /* Delete the marked fields.  */
   i = 0;
-  field_elem = rec_record_first_field (record);
-  while (rec_record_elem_p (field_elem))
+
+  iter = rec_mset_iterator (rec_record_mset (record));
+  while (rec_mset_iterator_next (&iter, MSET_FIELD, (const void**) &field, &elem))
     {
       if (deletion_mask[i])
         {
           if (recset_action == RECSET_ACT_COMMENT)
             {
-              comment = rec_field_to_comment (rec_record_elem_field (field_elem));
-              comment_elem = rec_record_elem_comment_new (record, comment);
-              rec_record_insert_after (record, field_elem, comment_elem);
-            }
+              /* Turn the field into a comment.  */
 
-          field_elem = rec_record_remove_field (record, field_elem);
-        }
-      else
-        {
-          field_elem = rec_record_next_field (record, field_elem);
+              comment = rec_field_to_comment (field);
+              rec_field_destroy (field);
+              rec_mset_elem_set_data (elem, (void *) comment);
+              rec_mset_elem_set_type (elem, MSET_COMMENT);
+            }
+          else
+            {
+              /* Remove the field from the list and dispose it.  */
+              
+              rec_mset_remove_elem (rec_record_mset (record), elem);
+            }
         }
 
       i++;
     }
+
+  rec_mset_iterator_free (&iter);
 }
 
 int
