@@ -43,6 +43,8 @@
 #include <sys/stat.h>
 #include <readline.h>
 #include <regex.h>
+#include <stdint.h>
+#include <time.h>
 
 #include <rec.h>
 #include <recutl.h>
